function [I]=FETToy(t,tSi,NSD,epsr,mt,T,VI,VF,NV,Ef,alphag,alphad,fb_flag,sg_flag);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Name: FETToy                                                                    %% 
%% Written by Anisur Rahman, Purdue University, Dec 24, 2002                       %%
%% email: rahmana@purdue.edu                                                       %%
%% Routines used: input.m, plot_output.m, fermi.m, Uscf_zero.m, write_results.m    %%
%%                                                                                 %%
%% Reference:                                                                      %%
%% [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,                              %%
%% "Theory of Ballistic Nanotransistors", to appear in IEEE TED, 2003.             %%
%%                                                                                 %%
%% Analytically calculates the ballistic I-V of a Double Gate ultra thin body      %%
%% MOSFET assuming that only the lowest unprimed subband is occupied (All          %%
%% constants are in MKS unit except energy, which is in eV)                        %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Physical constants
m0=0.91e-30;    hbar=1.05e-34;      q=1.6e-19;    eps0=8.85e-12;     kB=1.38e-23; 

m=mt*m0;                            % Transverse mass of electron
NSD2D=NSD*tSi;                      % 2D doping density in source and drain
kT=kB*T/q;                          % Thermal voltage.
V=linspace(VI,VF,NV);               % Voltage (gate or drain) steps.
Cins=epsr*eps0/t;                   % Gate insulator capacitance (F/m^2).
CG=2*Cins;
C_SIG=CG/alphag;                    % C_SIG=sum of capacitors (see eq. (7b) in [1]).  
U0=q/C_SIG;                         % Charging energy (eq. (8b) in [1]).

N2D=(2*m/(pi*hbar^2))*kT*q;         % Effective 2D density of states.
Ec_S0=Ef-kT*log(exp(NSD2D/N2D)-1);    % Equilibrium position of subband band edge in source.
N0=N2D*log(1+exp(Ef/kT));           % Electron concentration at the top of the barrier in neutral device.

N=zeros(NV,NV);                     % Mobile charge density.
I=zeros(NV,NV);                     % Current.
Ef_mat=zeros(NV,NV);                % Source Fermi level.   
Esub_max=zeros(NV,NV);              % Energy at the top of the barrier.
        
for kVg=1:NV                                                        % Bias loop begins.
    for kV=1:NV
        NSDcalc=0;
        dmu1=0;
        if fb_flag==1                                               % The case with floating source potental.
            while NSDcalc<NSD2D                                     % This loop ensures the charge neutrality in source 
                                                                    % extension (Sec. IVb of [1]).
                Vg=V(kVg);  Vd=V(kV);
                mu1=Ef+dmu1;   mu2=mu1-Vd;                          % Source and drain fermi levels.
                UL=-(alphag*Vg)-(alphad*Vd);                        
                % "UL" is the Laplace potential at the top of the barrier (Equation (7a) in [1]).
                % When the source potential is allowed to float to ensure charge neutrality in source extension,
                % an additional term "-(alphas*Vs)" is also present in RHS of this expression. Here we have assumed that
                % the source is still grounded, Vs=0, and the source fermi level Ef floats up to ensure charge neutrality.
                Uscf=fzero(@Uscf_zero,0,optimset('tolx',1e-12),N2D,mu1,mu2,kT,UL,U0,N0); 
                % "Uscf" is the self-consistent potential at the top of the barrier.
                % Equation (8a)in [1] can be written as f(Uscf)=0, expressed in subroutine Uscf_zero.m. Please note, 
                % from eqs. (4a),(4b) and (7c), that del_N is a function of Uscf too. The solution of it is evaluated here.
                EbarS=Uscf-Ec_S0;                                   % Source-channel barrier height.
                % The three carrier populations in eq. (A10) are calculated below.
                N1S=N2D*log((1+exp((mu1-Ec_S0)/kT))/(1+exp((mu1-Uscf)/kT)));
                N2S=0.5*N2D*log(1+exp((mu1-Uscf)/kT));
                N3S=0.5*N2D*log(1+exp((mu2-Uscf)/kT));
                NSDcalc=N1S+N2S+N3S;                                % Charge density in source.
                dmu1=dmu1+0.5e-3;                                   % Source Ef floated up to ensure charge neutrality.
            end
        else                                                        % The case without floating source potental.
            Vg=V(kVg);  Vd=V(kV);
            mu1=Ef+dmu1;   mu2=mu1-Vd;                              % Source and drain fermi levels.
            UL=-(alphag*Vg)-(alphad*Vd);                            % Laplace potential.
            Uscf=fzero(@Uscf_zero,0,optimset('tolx',1e-12),N2D,mu1,mu2,kT,UL,U0,N0); 
            % Please note the comments above in the case "fb_flag=1", for similar expression.
        end        
        dN=0.5*N2D*log((1+exp((mu1-Uscf)/kT))*(1+exp((mu2-Uscf)/kT)))-N0;   % Mobile charge induced by gate and drain 
                                                                            % bias (Sec. III in [1]).
        N(kV,kVg)=dN;
        eta1=(mu1-Uscf)/kT; eta2=(mu2-Uscf)/kT;
        % Vd changes along fixed column and Vg changes along fixed row.
        I(kV,kVg)=(q/hbar^2)*sqrt(2*m)*(q*kT/pi)^1.5*(fermi(eta1,1,1/2)-fermi(eta2,1,1/2));
        Esub_max(kV,kVg)=Uscf; Ef_mat(kV,kVg)=mu1; 
    end
end                                                             % Bias loop ends.

% convert the current into current per unit gate width %%%%%%%%%%%%%%%%%% added by Jing Wang 09/09/2003
if sg_flag==1
    I=I./2.0;
    disp('Current Converted for the Comparison with Single-Gate/Bulk MOSFETs');
    disp(blanks(1)');
end

% Transconductance, gm and output conductance, gd and vinjare calculated at highest gate and drain biases
gm=(I(NV,NV)-I(NV,NV-1))/(V(NV)-V(NV-1));
gd=(I(NV,NV)-I(NV-1,NV))/(V(NV)-V(NV-1));
Av=gm/gd;
vinj=I(NV,NV)/(q*N(NV,NV));

% Calculate S and DIBL  %%%%%%%%%%%%%%%%%% added by Jing Wang 09/09/2003
Ie1=log10(I(NV,:));
Ie2=log10(I(2,:));
vv1=interp1(Ie1,V,log10(I(NV,1)),'spline');
vv2=interp1(Ie1,V,log10(2*I(NV,1)),'spline');
vv3=interp1(Ie2,V,log10(I(NV,1)),'spline');
S=(vv2-vv1)/log10(2)*1000;
DIBL=(vv3-vv1)/(V(NV)-V(2))*1000;

plot_output;                                                    % Plot results.
write_results;                                                  % Write results in results.m file

