% FETToy (Toy model to simulate I-V characteristics of nanoscale double gate MOSFETs)
%
%
% Input
% -----
% Device:  Gate Oxide Thickness, t (m)
%          Silicon Body Thickness, tSi (m)
%          Doping Density in Source and Drain, NSD (1/m^3)
% Material:  Oxide Dielectric Constant, epsr
%            Transverse Effective Mass, mt
%            Temperature (K)
% Terminal Voltage:  Number of Bias Points, NV
%                    Voltage Range, VI,VF (V)
% Analytical Model:  Source Fermi Level, Ef (eV)
%                    Get Control Parameter, alphag
%                    Drain Control Parameter, alphad
%                    Floating Boundary, fb_flag
%
% Output
% ------
% Text file containg the results and the input parameters used in the calculations
% Plot showing the drain current vs. the drain voltage for different gate voltages
% Linear plot showing the drain current vs. the gate voltage for two different drain voltages
% Logarithmic plot showing the drain current vs. the gate voltage for two different drain voltages
%

clc;
clear all;
close all;



% -----------------
% Input Parameters
% -----------------

% Device:  Gate Oxide Thickness, t (m)
%
t = [];
while isempty(t)
	disp('Device:  Gate Oxide Thickness (m)');
	t = input('t = ');
	if isempty(t)
		t = 1.5e-9; % Default value
		disp('Using default value of "1.5e-9".');
	end
	disp(blanks(1)');
end

% Device:  Silicon Body Thickness, tSi (m)
%
tSi = [];
while isempty(tSi)
	disp('Device:  Silicon Body Thickness (m)');
	tSi = input('tSi = ');
	if isempty(tSi)
		tSi = 1.5e-9; % Default value
		disp('Using default value of "1.5e-9".');
	end
	disp(blanks(1)');
end

% Device:  Doping Density in Source and Drain, NSD (1/m^3)
%
NSD = [];
while isempty(NSD)
	disp('Device:  Doping Density in Source and Drain (1/m^3)');
	NSD = input('NSD = ');
	if isempty(NSD)
		NSD = 1e26; % Default value
		disp('Using default value of "1e26".');
	end
	disp(blanks(1)');
end

% Material:  Oxide Dielectric Constant, epsr
%
epsr = [];
while isempty(epsr)
	disp('Material:  Oxide Dielectric Constant');
	epsr = input('epsr = ');
	if isempty(epsr)
		epsr = 3.9; % Default value
		disp('Using default value of "3.9".');
	end
	disp(blanks(1)');
end

% Material:  Transverse Effective Mass, mt
%
mt = [];
while isempty(mt)
	disp('Material:  Transverse Effective Mass');
	mt = input('mt = ');
	if isempty(mt)
		mt = 0.19; % Default value
		disp('Using default value of "0.19".');
	end
	disp(blanks(1)');
end

% Material:  Temperature (K)
%
T = [];
while isempty(T)
	disp('Material:  Temperature (K)');
	T = input('T = ');
	if isempty(T)
		T = 300; % Default value
		disp('Using default value of "300".');
	end
	disp(blanks(1)');
end

% Terminal Voltage:  Number of Bias Points, NV
%
NV = [];
while isempty(NV)
	disp('Terminal Voltage:  Number of Bias Points');
	NV = input('NV = ');
	if isempty(NV)
		NV = 13; % Default value
		disp('Using default value of "13".');
	end
	disp(blanks(1)');
end

% Terminal Voltage:  Voltage Range, VI,VF (V)
%
VI = [];
VF = [];
while isempty(VI) | isempty(VF)
	disp('Terminal Voltage:  Voltage Range (V)');
	VI = input('(Initial)  VI = ');
	if isempty(VI)
		VI = 0; % Default value
		disp('Using default value of "0".');
	end
	VF = input('(Final)  VF = ');
	if isempty(VF)
		VF = 0.6; % Default value
		disp('Using default value of "0.6".');
	end
	disp(blanks(1)');
end

% Analytical Model:  Source Fermi Level, Ef (eV)
%
Ef = [];
while isempty(Ef)
	disp('Analytical Model:  Source Fermi Level (eV)');
	Ef = input('Ef = ');
	if isempty(Ef)
		Ef = -0.32; % Default value
		disp('Using default value of "-0.32".');
	end
	disp(blanks(1)');
end

% Analytical Model:  Get Control Parameter, alphag
%
alphag = [];
while isempty(alphag)
	disp('Analytical Model:  Gate Control Parameter');
	alphag = input('alphag = ');
	if isempty(alphag)
		alphag = 0.88; % Default value
		disp('Using default value of "0.88".');
	end
	disp(blanks(1)');
end

% Analytical Model:  Drain Control Parameter, alphad
%
alphad = [];
while isempty(alphad)
	disp('Analytical Model:  Drain Control Parameter');
	alphad = input('alphad = ');
	if isempty(alphad)
		alphad = 0.035; % Default value
		disp('Using default value of "0.035".');
	end
	disp(blanks(1)');
end

% Analytical Model:  Floating Boundary, fb_flag
%
fb_flag = [];
while isempty(fb_flag)
	disp('Analytical Model:  Floating Boundary');
	fb_flag = input('(0=no/1=yes)  fb_flag = ');
	if isempty(fb_flag)
		fb_flag = 0; % Default value
		disp('Using default value of "0".');
	end
	disp(blanks(1)');
end

% Analytical Model:  Current Conversion (divided by 2), sg_flag
%
sg_flag = [];
while isempty(sg_flag)
	disp('Analytical Model:  Convert Current for the comparison with single-gate/bulk MOSFETs');
	sg_flag = input('(0=no/1=yes)  sg_flag = ');
	if isempty(sg_flag)
		sg_flag = 0; % Default value
		disp('Using default value of "0".');
	end
	disp(blanks(1)');
end


% Call Main Program
% -----------------

I=FETToy(t,tSi,NSD,epsr,mt,T,VI,VF,NV,Ef,alphag,alphad,fb_flag,sg_flag);
