function [I] = SiNWFETToy( t,d,epsr,mt,degen,T, VI,VF,NV, Ef,alphag,alphad,Rs )

% function [I,V,Uscf,N] = SiNWFETToy( t,d,epsr,mt,T, VI,VF,NV, Ef,alphag,alphad )
%    Inputs:
%   ----------------
%       t - insulator thickness (m)
%       d - NT diameter (m)
%       epsr - insulator dielectric constant
%       mt - Transport Effective mass
%       T - Temp (K)
%
%       VI - Initial Voltage
%       VF - Final Voltage
%       NV - # of bias points
%
%       Ef - Fermi Level
%       alphag - gate control parameter
%       alphad - drain control parameter
%
%   Outputs:
%   ------------------
%       I = Current
%       V = Voltage
%       Uscf = Self Consistent Potential 
%       N = Free charge
%
% Based on FETToy Originally developed by : Anisur Rahman
%   Reference:
%   [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,
%   "Theory of Ballistic Nanotransistors", to appear in IEEE TED, 2003.
%
% Adapted for SiNWFETToy by : Jing Wang
% Latest Version Updated by : Sayed Hasan (05/24/2004)
%--------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Original Note by Anisur Rahman                                                  %% 
%%---------------------------------------------------------------------------------%% 
%% Name: FETToy                                                                    %% 
%% Written by Anisur Rahman, Purdue University, Dec 24, 2002                       %%
%% email: rahmana@purdue.edu                                                       %%
%% Routines used: input.m, plot_output.m, fermi.m, Uscf_zero.m, write_results.m    %%
%%                                                                                 %%
%% Reference:                                                                      %%
%% [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,                              %%
%% "Theory of Ballistic Nanotransistors", to appear in IEEE TED, 2003.             %%
%%                                                                                 %%
%% Analytically calculates the ballistic I-V of a Double Gate ultra thin body      %%
%% MOSFET assuming that only the lowest unprimed subband is occupied (All          %%
%% constants are in MKS unit except energy, which is in eV)                        %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Physical constants
m0=0.91e-30;    hbar=1.05e-34;      q=1.6e-19;    eps0=8.85e-12;     kB=1.38e-23; 

m=mt*m0;                            % Transverse mass of electron
kT=kB*T/q;                          % Thermal voltage.
V=linspace(VI,VF,NV);               % Voltage (gate or drain) steps.
Cins=2*pi*epsr*eps0/log((t+d/2)/(d/2)); % Capacitance per unit length
CG=Cins;
C_SIG=CG/alphag;                    % C_SIG=sum of capacitors (see eq. (7b) in [1]).  
U0=q/C_SIG;                         % Charging energy (eq. (8b) in [1]).

% degeneracy is modified by Sayed Hasan (05/26/2004)
N1D= degen*m/hbar*(2*kT*q/pi/m)^0.5;         % Effective 1D density of states (include both +k and -k, valley degeneracy is degen)
% End degen. modification

N0 = N1D*fermi(Ef/kT,1,-1/2);         % Electron concentration at the top of the barrier in neutral device.
I0 = degen*(q*kB*T/pi/hbar);          % Valley degeneracy is given as input in "degen"

N=zeros(NV,NV);                     % Mobile charge density.
I=zeros(NV,NV);                     % Current.
Ef_mat=zeros(NV,NV);                % Source Fermi level.   
Esub_max=zeros(NV,NV);              % Energy at the top of the barrier.

for kVg=1:NV                                                        % Bias loop begins.
    for kV=1:NV
        Vg=V(kVg);  Vd=V(kV);
        mu1=Ef;   mu2=mu1-Vd;                                   % Source and drain fermi levels.
        UL=-(alphag*Vg)-(alphad*Vd);                            % Laplace potential.
        Uscf=fzero(@Uscf_zero,0,optimset('tolx',1e-12),N1D,mu1,mu2,kT,UL,U0,N0);
        dN=0.5*N1D*(fermi((mu1-Uscf)/kT,1,-1/2)+fermi((mu2-Uscf)/kT,1,-1/2))-N0;   % Mobile charge induced by gate and drain 
                                                                            
        N(kV,kVg)=dN;
        eta1=(mu1-Uscf)/kT; eta2=(mu2-Uscf)/kT;
        % Vd changes along fixed column and Vg changes along fixed row.
        I(kV,kVg)=I0*(fermi(eta1,1,0)-fermi(eta2,1,0)); 
        Esub_max(kV,kVg)=Uscf; Ef_mat(kV,kVg)=mu1; 
        
        % Added by Sayed Hasan 5/26/2004
        Us(kV,kVg) = Uscf;

        % Calculate Quantum Capacitance and average velocity %%%%%%%%%%%%%%%%%% added by Sayed Hasan 05/24/2004
        if ((kV==2)|(kV==NV)),
            deltaU = 0.001*kT;
            if (kV==2),
                N_U2 = 0.5*N1D*(fermi((mu1-Uscf+deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf+deltaU/2)/kT,1,-1/2))-N0;                
                N_U1 = 0.5*N1D*(fermi((mu1-Uscf-deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf-deltaU/2)/kT,1,-1/2))-N0;                
                CQ(1, kVg) = q*(N_U2-N_U1)/deltaU;
            elseif (kV==NV),
                N_U2 = 0.5*N1D*(fermi((mu1-Uscf+deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf+deltaU/2)/kT,1,-1/2))-N0;                
                N_U1 = 0.5*N1D*(fermi((mu1-Uscf-deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf-deltaU/2)/kT,1,-1/2))-N0;                
                CQ(2, kVg) = q*(N_U2-N_U1)/deltaU;
                v_ave(kVg)=I(kV,kVg)/q/(dN+N0);
            end
        end
    end
end                                                             % Bias loop ends.


% Transconductance, gm and output conductance, gd and vinjare calculated at highest gate and drain biases
gm=(I(NV,NV)-I(NV,NV-1))/(V(NV)-V(NV-1));
gd=(I(NV,NV)-I(NV-1,NV))/(V(NV)-V(NV-1));
Av=gm/gd;
vinj=I(NV,NV)/(q*N(NV,NV));

% Calculate S and DIBL  %%%%%%%%%%%%%%%%%% added by Jing Wang 09/09/2003
Ie1=log10(I(NV,:));
Ie2=log10(I(2,:));
vv1=interp1(Ie1,V,log10(I(NV,1)),'spline');
vv2=interp1(Ie1,V,log10(2*I(NV,1)),'spline');
vv3=interp1(Ie2,V,log10(I(NV,1)),'spline');
S=(vv2-vv1)/log10(2)*1000;
DIBL=(vv3-vv1)/(V(NV)-V(2))*1000;

if (Rs>0)
I=treatRs(V',V',I',Rs);     %%%%%%%%%%%%%%%% added by Yang Liu 10/19/2005
end

%%%%%%%%%%%%%%%  OUTPUT (Sayed Hasan: 5/25/2004)  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plot_output;            % Plot results
save_charge;            % Save q*N
save_current;           % Save I
save_uscf;              % Save Us
write_results;          % Write results in results.m file
save rawdata;           % save matlab variables
