% This function evaluates the Fermi-Dirac integral of order fermi-order.
% The Fermi-Dirac integral is in the form defined by Blakemore in
% "Semiconductor Stastics".
% x is the argument of the Fermi-Dirac integral in units of kT.
% fermi-flag=1 for degeneracy and 0 for the non-degenerate approximation.
% fermi_order is 1/2 or 0.

% The empirical expressions of
%   J. S. Blakemore, "Approximations for the Fermi-Dirac integrals,
%   specially the function, F_1/2(eta), used to describe electron density
%   in a semiconductor," Solid-State Electron., Vol. 25, pp. 1067-1076,
%   1982 are used.

function [y]=fermi(x,fermi_flag,fermi_order)

if fermi_order==1/2
    if fermi_flag==1
        exp_fac=exp(-0.17*(x+1.0).^2);
        nu=x.^4+50.0+33.6*x.*(1.0-0.68*exp_fac);
        zeta=3.0*sqrt(pi)./(4.0*nu.^0.375);
        y=exp(x)./(1.0+zeta.*exp(x));
    elseif fermi_flag==0
        y=exp(x);
    end
elseif fermi_order==0

    if fermi_flag==1
        y=log(1+exp(x));
    elseif fermi_flag==0
        y=exp(x);
    end
end
