function [I,V,Uscf,N]=FETToy(sdg_flag,t,tSi,NSD,epsr,mt,Vdegen,T, NV,VI,VF,Ef,alphag,alphad,fb_flag,Rs,intext);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Name: FETToy                                                                    %% 
%% Written by Anisur Rahman, Purdue University, Dec 24, 2002                       %%
%% email: rahmana@purdue.edu                                                       %%
%% Revised and improved by Jing Wang, Sayed Hasan, Aman, Akira and Yang Liu.       %%
%% Final updated on Sep 25, 2005                                                   %%
%%                                                                                 %%
%% Routines used: input.m, plot_output.m, fermi.m, Uscf_zero.m, write_results.m    %%
%%                                                                                 %%
%% Reference:                                                                      %%
%% [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,                              %%
%% "Theory of Ballistic Nanotransistors", Invited paper,TED,Sept. 2003, Vol. 50,   %%
%% No. 9, pp. 1853-1864                                                            %%
%% Analytically calculates the ballistic I-V of a ultra thin body                  %%
%% MOSFET assuming that only the lowest unprimed subband is occupied (All          %%
%% constants are in MKS unit except energy, which is in eV)                        %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global UL
global N2D
global mu1
global mu2
global kT
global UC
global N0

% Physical constants
m0=0.91e-30;    hbar=1.05e-34;      q=1.6e-19;    eps0=8.85e-12;     kB=1.38e-23; 

meff=mt*m0;                         % Transport effecitve mass of electron.
NSD2D=NSD*tSi;                      % 2D doping density in source and drain
kT=kB*T/q;                          % Thermal voltage.
V=linspace(VI,VF,NV);               % Voltage (gate or drain) steps.

Cins=epsr*eps0/t;                   % Capacitance per gate(F/m^2).
CG=sdg_flag*Cins;                   % Total gate capacitance, note sdg_flag equals 1 for single and 2 for double gate.
C_SIG=CG/alphag;                    % C_SIG=sum of capacitance (see eq. (7b) in [1]).  
UC=q/C_SIG;                         % Charging energy (eq. (8b) in [1]).

N2D=Vdegen*(meff/(pi*hbar^2))*kT*q; % Effective 2D density of states (Vdegen is the valley degeneracy, the default value is 2 for Si).
Ec_S0=Ef-kT*log(exp(NSD2D/N2D)-1);  % Equilibrium position of subband band edge in source (note only the lowest subband is occupied). 
N0=N2D*log(1+exp(Ef/kT));           % Electron concentration at the top of the barrier in neutral device (Equation (3) of [1]).

N=zeros(NV,NV);                     % Mobile charge density.
I=zeros(NV,NV);                     % Current.
Ef_mat=zeros(NV,NV);                % Source Fermi level.   
Esub_max=zeros(NV,NV);              % Energy at the top of the barrier.

for kVg=1:NV                                                        % Bias loop begins. kVg is the index for the gate voltage.
    for kV=1:NV                                                     % kV is the index for the drain bias.
        NSDcalc=0;                                                  % Calculated doping density in source.
        dmu1=0;
        if fb_flag==1                                               % The case with floating source potental.
            while NSDcalc<NSD2D                                     % This loop ensures the charge neutrality in source 
                                                                    % extension (Sec. IVb of [1]).
                Vg=V(kVg);  Vd=V(kV);
                mu1=Ef+dmu1;   mu2=mu1-Vd;                          % Source and drain fermi levels.
                UL=-(alphag*Vg)-(alphad*Vd);                        
                % "UL" is the Laplace potential at the top of the barrier (Equation (7a) in [1]).
                % When the source potential is allowed to float to ensure charge neutrality in source extension,
                % an additional term "-(alphas*Vs)" is also present in RHS of this expression. Here we have assumed that
                % the source is still grounded, Vs=0, and the source fermi level Ef floats up to ensure charge neutrality.
#                Uscf=fzero(@Uscf_zero,0,optimset('tolx',1e-12),N2D,mu1,mu2,kT,UL,UC,N0); 
                Uscf=fzero(@Uscf_zero,[-100,100],optimset('tolx',1e-12),N2D,mu1,mu2,kT,UL,UC,N0); 
                % "Uscf" is the self-consistent potential at the top of the barrier.
                % Equation (8a)in [1] can be written as f(Uscf)=0, expressed in subroutine Uscf_zero.m. Please note, 
                % from eqs. (4a),(4b) and (7c), that del_N is a function of Uscf too. The solution of it is evaluated here.
                EbarS=Uscf-Ec_S0;                                   % Source-channel barrier height.
                % The three carrier populations in eq. (A10) are calculated below.
                N1S=N2D*log((1+exp((mu1-Ec_S0)/kT))/(1+exp((mu1-Uscf)/kT)));
                N2S=0.5*N2D*log(1+exp((mu1-Uscf)/kT));
                N3S=0.5*N2D*log(1+exp((mu2-Uscf)/kT));
                NSDcalc=N1S+N2S+N3S;                                % Charge density in source.
                dmu1=dmu1+0.5e-3;                                   % Source Ef floated up to ensure charge neutrality.
            end
        else                                                        % The case without floating source potental.
            Vg=V(kVg);  Vd=V(kV);
            mu1=Ef+dmu1;   mu2=mu1-Vd;                              % Source and drain fermi levels.
            UL=-(alphag*Vg)-(alphad*Vd);                            % Laplace potential.
#            Uscf=fzero(@Uscf_zero,0,optimset('tolx',1e-12),N2D,mu1,mu2,kT,UL,UC,N0); 
            Uscf=fzero(@Uscf_zero,[-100,100],optimset('tolx',1e-12),N2D,mu1,mu2,kT,UL,UC,N0); 
            % Please note the comments above in the case "fb_flag=1", for similar expression.
        end        
        dN=0.5*N2D*log((1+exp((mu1-Uscf)/kT))*(1+exp((mu2-Uscf)/kT)))-N0;   % Mobile charge induced by gate and drain 
                                                                            % bias (Sec. III in [1]).
        N(kV,kVg)=dN;                                                       % Mobile charge density.
        eta1=(mu1-Uscf)/kT; eta2=(mu2-Uscf)/kT;
        
        % The net current (see eq.(9) and eq.(A7) in [1]) is calculated below. Note the valley degeneracy is included in the 
        % effective 2D DOS N2D. Vd changes along fixed column and Vg changes along fixed row.
        I(kV,kVg)=1/2*q*N2D*sqrt(2*kT*q/(pi*meff))*(fermi(eta1,1,1/2)-fermi(eta2,1,1/2)); 
        Esub_max(kV,kVg)=Uscf; Ef_mat(kV,kVg)=mu1;                          % Energy at the top of the barrier and the source 
                                                                            % Fermi level. 
        Us(kV,kVg) = Uscf;  % The self-consistent potential at each bias.

        % The quantum capacitance is calculated below, at low and high drain biases, following eq.(2) in [1]: CQ = d(qN)/d(-Uscf/q).
        % Numerically the quantum capacitance is achieved as CQ=q*(N(Uscf-deltaU/2)-N(Uscf+deltaU/2))/deltaU. 
        if ((kV==2)|(kV==NV)),
            deltaU = 0.002*kT;  % The increment of Uscf is +(-)deltaU/2.
            if (kV==2),         % Low drain bias.
                N_U2 = 0.5*N2D*log((1+exp((mu1-Uscf+deltaU/2)/kT))*(1+exp((mu2-Uscf+deltaU/2)/kT)))-N0;  % N_U2 = (N1+N2)-N0 at Uscf2 = Uscf-deltaU/2.
                N_U1 = 0.5*N2D*log((1+exp((mu1-Uscf-deltaU/2)/kT))*(1+exp((mu2-Uscf-deltaU/2)/kT)))-N0;  % N_U1 = (N1+N2)-N0 at Uscf1 = Uscf+deltaU/2.
                CQ(1, kVg) = q*(N_U2-N_U1)/deltaU;  % Quantum Capacitance at low drain bias.
            elseif (kV==NV),    % High drain bias.
                N_U2 = 0.5*N2D*log((1+exp((mu1-Uscf+deltaU/2)/kT))*(1+exp((mu2-Uscf+deltaU/2)/kT)))-N0;
                N_U1 = 0.5*N2D*log((1+exp((mu1-Uscf-deltaU/2)/kT))*(1+exp((mu2-Uscf-deltaU/2)/kT)))-N0;
                CQ(2, kVg) = q*(N_U2-N_U1)/deltaU;  % Quantum Capacitance at high drain bias.
                v_ave(kVg)=I(kV,kVg)/(q*(dN+N0));   % Injection velocity (see eq.(A8) in [1]).
            end
        end
    end
end                                                             % Bias loop ends.


% Transconductance, gm and output conductance, gd and injection velocity vinj are calculated at highest gate and drain biases as below.
gm=(I(NV,NV)-I(NV,NV-1))/(V(NV)-V(NV-1));               % Transconductance gm = d(I)/d(Vg) with fixed Vd.
gd=(I(NV,NV)-I(NV-1,NV))/(V(NV)-V(NV-1));               % Output conductance gd = d(I)/d(Vd) with fixed Vg.
Av=gm/gd;                                               % Voltage gain at highest gate and drain biases.
I1=1/2*q*N2D*sqrt(2*kT*q/(pi*meff))*fermi(eta1,1,1/2);  % Source current (see eq.(A7) in [1]).
N1=1/2*N2D*log((1+exp((mu1-Uscf)/kT)));                 % Source electron density (see eq.(4a) in [1]).
vinj=I1/(q*N1);                                         % Injection velocity (see eq.(A8) in [1]).

% The subthreshold swing, S and the drain-induced barrier lowering, DIBL are calculated below.
Ie1=log10(I(NV,:));                           % log10(I) for the highest drain Vd, used to evaluate S numerically.                            
vv1=interp1(Ie1,V,log10(I(NV,1)),'spline');   % vv1 is the corresponding Vg when I = I(Nv,1);
vv2=interp1(Ie1,V,log10(2*I(NV,1)),'spline'); % vv2 is the corresponding Vg when I = 2*I(Nv,1);
S=(vv2-vv1)/log10(2)*1000;                    % S is evaluated at low Vg and highest Vd, with the definition that S is the inverse slope of the
                                              % log10(I) vs. Vg curve, [mVolt/decade].
Ie2=log10(I(2,:));                            % log10(I) for the lowest drain bias Vd, used to evaluate DIBL numerically.
vv3=interp1(Ie2,V,log10(I(NV,1)),'spline');   % vv3 is the corresponding Vg in low drain bias when I = I(NV,1) at the highest drain bias.
DIBL=(vv3-vv1)/(V(NV)-V(2))*1000;             % DIBL is evaluated at the highest and lowest Vd at low gate bias Vg, [mVolt/Volt]

% Series resistance Rs is treated below.
I=treatRs(V',V',I,Rs,intext);

%%=================================  OUTPUT  =================================%%

%plot_output;            % Plot results

%%%%%%%%%%%%%%%%%%%% Dump output data %%%%%%%%%%%%%%%%%%%%%%%%
save_charge;            % Save q*N
save_current;           % Save I
save_gm;                % Save gm values
save_uscf;              % Save Us
save_CQ;                % Save CQ
save_v_ave;             % Save v_ave
write_results;          % Write results in results.m file
%save rawdata;           % save matlab variables
