% FETToy (Toy model to simulate I-V characteristics of Si nanoscale MOSFETs)
%
% Input
% -----
% Device Specification: 
%          1) Single Gate / Double Gate, sdg_flag 
%          2) Gate Oxide Thickness, t (m)
%          3) Si Body Thickness, tSi (m)
%          4) Doping Density in Source and Drain, NSD (1/m^3)
%
% Material Specification:
%          1) Gate Oxide Dielectric Constant, epsr
%          2) Transport Effective Mass, mt
%          3) Valley degeneracy, Vdegen
%          4) Temperature, T (K)
%
% Terminal Voltage:  
%          1) Number of Bias Points, NV
%          2) Voltage Range, VI,VF (V)
%
% Analytical Model:  
%          1) Source Fermi Level, Ef (eV)
%          2) Gate Control Parameter, alphag
%          3) Drain Control Parameter, alphad
%          4) Floating Boundary, fb_flag
%          5) Series Resistance, Rs (ohm)
%          6) int/ext specification
%
% Output
% ------
% 1.  Text files: 
%       A. current_data.txt
%       B. results.txt
%
% 2.  Following plots:
%       A. Id vs. Vgs At 2nd and Last Drain bias (Smilog) : Filename:  Id_vs_Vgs_log.ps
%       B. Id vs. Vgs At 2nd and Last Drain bias (Linear) : Filename:  Id_vs_Vgs_lin.ps
%       C. Id vs. Vds At Different Vgs (Linear)           : Filename:  Id_vs_Vds.ps
%       D. Mobile at top of the barrier vs. Vds @ Different Vgs : Filename:  N_vs_Vds.ps 
%       E. Quantum Capacitance vs. Vgs at Maximum Vds     : Filename:  CQ_vs_Vgs.ps
%       F. Average Velocity vs. Vgs at Maximum Vds        : Filename:  Velocity_vs_Vgs.ps
%
% 3.  Matlab Rawdata: 
%       A. Rawdata.mat
%
% Interface Script Written by : Sayed Hasan (05/24/2004)
%--------------------------------------------------------------------------

clc;
clear all;
close all;

% -----------------
% Input Parameters
% -----------------

fprintf('Si NanoScale MOSFET Simulation ...\n');

% Device Specification:  
%==========================================================================
		fprintf('Device Specifications:\n');
		fprintf('======================\n');
        
		%% 1) Single Gate / Double Gate, sdg_flag
		sdg_flag = [];
		while isempty(sdg_flag)
			fprintf('   Double Gate / Single Gate MOSFET:   \n');
            fprintf('       1) Single Gate:\n');
            fprintf('       2) Double Gate:\t');
			sdg_flag = input('sdg_flag = ');
			if isempty(sdg_flag)
				sdg_flag = 1; % Default value (single gate)
				fprintf('\b\b 1 (Using Default ...)\n');
            end
            fprintf('\n');
        end

        %% 2) Gate Oxide Thickness, t (m)
		t = [];
		while isempty(t)
			fprintf('   Gate Insulator Thickness (m):       ');
			t = input('t = ');
			if isempty(t)
				t = 1.5e-9; % Default value
				fprintf('\b\b 1.5e-9 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
        %% 3) Si Body Thickness, tSi (m)
        tSi = [];
        while isempty(tSi)
            fprintf('   Si Body Thickness (m):              ');
            tSi = input('tSi = ');
            if isempty(tSi)
                tSi = 1.0e-9; % Default value
                fprintf('\b\b 1.0e-9 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
        %% 4) Doping Density in Source and Drain, NSD (1/m^3)
        NSD = [];
        while isempty(NSD)
	        disp('Device:  Doping Density in Source and Drain (1/m^3)');
	        NSD = input('NSD = ');
	        if isempty(NSD)
		        NSD = 1e26; % Default value
		        fprintf('\b\b 1e26 (Using default ...)\n');
            end
            fprintf('\n');
        end
        
% Material Specification:  
%==========================================================================
		fprintf('Mateiral Specifications:\n');
		fprintf('======================\n');
        
        %% 1) Gate Oxide Dielectric Constant, epsr
        epsr = [];
        while isempty(epsr)
            fprintf('   Gate Insulator Dielectric Const.:   ');
            epsr = input('epsr = ');
            if isempty(epsr)
                epsr = 3.9; % Default value
                fprintf('\b\b 3.9 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
        %% 2) Transport Effective Mass, mt
        mt = [];
        while isempty(mt)
            fprintf('   Transport Effective Mass:           ');
            mt = input('mt = ');
            if isempty(mt)
                mt = 0.19; % Default value
                fprintf('\b\b 0.19\n');
            end
            fprintf('\n');
        end
        
        %% 3) Valley Degeneracy, Vdegen
        Vdegen = [];
        while isempty(Vdegen)
            fprintf('   Valley Degeneracy:                  ');
            Vdegen = input('Vdegen = ');
            if isempty(Vdegen)
                Vdegen = 2; % Default value
                fprintf('\b\b 2 (Using Default ...)\n');
            end
            fprintf('\n');
        end

        %% 4) Temperature, T (K)
        T = [];
        while isempty(T)
            fprintf('   Temperature (K):                    ');
            T = input('T = ');
            if isempty(T)
                T = 300; % Default value
                fprintf('\b\b 300 (Using Default ...)\n');
            end
            fprintf('\n');
        end

% Terminal Voltage: 
%==========================================================================
		fprintf('Terminal Voltage:\n');
        fprintf('=================\n');
        
        %% 1) Number of Bias Points, NV
        NV = [];
        while isempty(NV)
            fprintf('   Number of Bias Points:  ');
            NV = input('NV = ');
            if isempty(NV)
                NV = 13; % Default value
                fprintf('\b\b 13 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
        %% 2) Voltage Range, VI,VF (V)
        VI = [];
        VF = [];
        while isempty(VI) | isempty(VF)
            fprintf('   Voltage Range (V):\n');
            fprintf('\t\t\t'); 
            VI = input('(Initial)  VI = ');
            if isempty(VI)
                VI = 0; % Default value
                fprintf('\b\b 0 (Using Default ...)\n');
            end
            fprintf('\t\t\t'); 
            VF = input('(Final)  VF = ');
            if isempty(VF)
                VF = 1.0; % Default value
                fprintf('\b\b 1.0 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
% Analytical Model:  
%==========================================================================
		fprintf('Analytical Model:\n');
        fprintf('=================\n');
        
        %% 1) Source Fermi Level, Ef (eV)
        Ef = [];
        while isempty(Ef)
            fprintf('   Source Fermi Level (eV):    ');
            Ef = input('Ef = ');
            if isempty(Ef)
                Ef = -0.32; % Default value
                fprintf('\b\b -0.32 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
		%% 2) Gate Control Parameter, alphag    
        alphag = [];
        while isempty(alphag)
            fprintf('   Gate Control Parameter:     ');
            alphag = input('alphag = ');
            if isempty(alphag)
                alphag = 0.88; % Default value
                fprintf('\b\b 0.88 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
        %% 3) Drain Control Parameter, alphad
        alphad = [];
        while isempty(alphad)
            fprintf('   Drain Control Parameter:    ');
            alphad = input('alphad = ');
            if isempty(alphad)
                alphad = 0.035; % Default value
                fprintf('\b\b 0.035 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
        %% 4) Floating Boundary, fb_flag
        fb_flag = [];
        while isempty(fb_flag)
	        fprintf('   Floating Boundary:    ');
	        fb_flag = input('(0=no/1=yes)  fb_flag = ');
	        if isempty(fb_flag)
		        fb_flag = 0; % Default value
		        fprintf('\b\b 0 (Using Default ...)\n');
            end
	        fprintf('\n');
        end
        
        %% 5) Series Resistance, Rs (ohm)
        Rs = [];
        while isempty(Rs)
            fprintf('   Series Resistance:    ');
            Rs = input('Rs = ');
            if isempty(Rs)
                Rs = 0; % Default value
                fprintf('\b\b 0 (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
        %% 6) int/ext specification
        intext = [];
        while isempty(intext)
            fprintf('   type int2ext or ext2int:    ');
            intext = input('intext = ');
            if isempty(intext)
                intext = 'int2ext'; % Default value
                fprintf('\b\b int2ext (Using Default ...)\n');
            end
            fprintf('\n');
        end
        
% Call Main Program
[I] = FETToy(sdg_flag,t,tSi,NSD,epsr,mt,Vdegen,T, NV,VI,VF,Ef,alphag,alphad,fb_flag,Rs,intext);