function [I] = CNTFETToy( t,d,epsr,T, NV_Gate,VI_Gate,VF_Gate,NV_Drain,VI_Drain,VF_Drain, Ef,alphag,alphad,Rs )

% function [I,V,Uscf,N] = CNTFETToy( t,d,epsr,T, VI,VF,NV,  Ef,alphag,alphad )
%    Inputs:
%   ----------------
%       t - insulator thickness (m)
%       d - NT diameter (m)
%       epsr - insulator dielectric constant
%       T - Temp (K)
%
%       VI_Gate/VI_Drain - Initial Gate/Drain Voltage
%       VF_Gate/VF_Drain - Final Gate/Drain Voltage
%       NV_Gate/NV_Drain - # of Gate/Drain bias points
%
%       Ef - Fermi Level
%       alphag - gate control parameter
%       alphad - drain control parameter
%
%   Outputs:
%   ------------------
%       I = Current
%       V = Voltage
%       Uscf = Self Consistent Potential 
%       N = Free charge
%
% Based on FETToy Originally developed by : Anisur Rahman
%   Reference:
%   [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,
%   "Theory of Ballistic Nanotransistors", to appear in IEEE TED, 2003.
%
% Adapted for CNTFETToy by : Jing Wang
% Updated by : Sayed Hasan (05/24/2004)
% Latest Version Updated by : Yang Liu (11/23/2005)
%--------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Original Note by Anisur Rahman                                                  %% 
%%---------------------------------------------------------------------------------%% 
%% Name: FETToy                                                                    %% 
%% Written by Anisur Rahman, Purdue University, Dec 24, 2002                       %%
%% email: rahmana@purdue.edu                                                       %%
%% Routines used: input.m, plot_output.m, fermi.m, Uscf_zero.m, write_results.m    %%
%%                                                                                 %%
%% Reference:                                                                      %%
%% [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,                              %%
%% "Theory of Ballistic Nanotransistors", to appear in IEEE TED, 2003.             %%
%%                                                                                 %%
%% Analytically calculates the ballistic I-V of a Double Gate ultra thin body      %%
%% MOSFET assuming that only the lowest unprimed subband is occupied (All          %%
%% constants are in MKS unit except energy, which is in eV)                        %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% FETToy for CNTFETs ...

% Physical constants
m0=0.91e-30;    hbar=1.05e-34;      q=1.6e-19;    eps0=8.85e-12;     kB=1.38e-23; 

kT=kB*T/q;                          % Thermal voltage.
V_Gate=linspace(VI_Gate,VF_Gate,NV_Gate);               % Gate voltage steps.
V_Drain=linspace(VI_Drain,VF_Drain,NV_Drain);               % Gate voltage steps.

Cins=2*pi*epsr*eps0/log((t+d/2)/(d/2));  % Capacitance per unit length
CG=Cins;
C_SIG=CG/alphag;                    % C_SIG=sum of capacitors (see eq. (7b) in [1]).  
U0=q/C_SIG;                         % Charging energy (eq. (8b) in [1]).

% for CNTFETs (calculate D0 and EG)
dd=d;                   % tube diameter
a_cc=1.42e-10;          % C-C bond length
t0=3.0;                 % Overlap Integral TB parameter
D0=8/(3*pi*a_cc*t0);    % DOS [1/(m*eV)], the DOS is expressed as: Dznt(E) == 4/(3*pi*t*a_cc)*(E/sqrt(E^2-Ev^2)),for up- and down- branches and +/-kx;
                        % here we should include the spin by times a factor of '2', 
                        % this is the constanct pre-factor of the DOS for a zigzag CNT, the valley degeneracy is not included so far.
EG=2*a_cc*t0/dd;        % Band Gap [eV], corresponding to the minimum bandgap of a semiconducting zigzag CNT when abs(v-2m/3)==1
N0=N_CNT(D0,EG,kT,Ef);  % Electron concentration at the top of the barrier in neutral device.
I0=(2*q*kB*T/pi/hbar);  % Current per unit energy; the factor '2' accounts for the valley degeneracy is 2

% Start Usual calculation
N=zeros(NV_Drain,NV_Gate);                     % Mobile charge density.
I=zeros(NV_Drain,NV_Gate);                     % Current.
Ef_mat=zeros(NV_Drain,NV_Gate);                % Source Fermi level.   
Esub_max=zeros(NV_Drain,NV_Gate);              % Energy at the top of the barrier.

% CQ=zeros(NV,1);         % Qauntum Capacitance
% v_ave=zeros(NV,1);      % average velocity


for kVg=1:NV_Gate            % Bias loop begins.
    Vg=V_Gate(kVg);  
    for kV=1:NV_Drain
        Vd=V_Drain(kV);
        mu1=Ef;   mu2=mu1-Vd;                                   % Source and drain fermi levels.
        UL=-(alphag*Vg)-(alphad*Vd);                            % Laplace potential.
        Uscf=fzero(@Uscf_zero,0,optimset('tolx',1e-12),D0,EG,mu1,mu2,kT,UL,U0,N0);
        
        fermi_flag=1; if (mu1-Uscf)/kT<-20, fermi_flag=0; end
        dN=N_CNT(D0/2,EG,kT,mu1-Uscf)+N_CNT(D0/2,EG,kT,mu2-Uscf)-N0;   % Mobile charge induced by gate and drain 
                                                         
        N(kV,kVg)=dN;
        eta1=(mu1-Uscf)/kT; eta2=(mu2-Uscf)/kT;
        
        % Vd changes along fixed column and Vg changes along fixed row.
        I(kV,kVg)=I0*(fermi(eta1,fermi_flag,0)-fermi(eta2,fermi_flag,0)); 
        Esub_max(kV,kVg)=Uscf; 
        Ef_mat(kV,kVg)=mu1; 
        
        % Added by Sayed Hasan 5/26/2004
        Us(kV,kVg) = Uscf;
        
        % Quantum Capacitance and velocity calculation added by Sayed Hasan 05/28/2004
        if ((kV==2)|(kV==NV_Drain)),
            deltaU = 0.002*kT;
            if (kV==2),
                N_U2 =  N_CNT(D0/2,EG,kT,mu1-Uscf+deltaU/2)+N_CNT(D0/2,EG,kT,mu2-Uscf+deltaU/2)-N0; 
                N_U1 =  N_CNT(D0/2,EG,kT,mu1-Uscf-deltaU/2)+N_CNT(D0/2,EG,kT,mu2-Uscf-deltaU/2)-N0; 
                CQ(1, kVg) = q*(N_U2-N_U1)/deltaU;
                %q*(N_CNT(D0/2,EG,kT,mu1-Uscf+0.001*kT)-N_CNT(D0/2,EG,kT,mu1-Uscf-0.001*kT))/(0.002*kT);
            elseif (kV==NV_Drain),
                N_U2 =  N_CNT(D0/2,EG,kT,mu1-Uscf+deltaU/2)+N_CNT(D0/2,EG,kT,mu2-Uscf+deltaU/2)-N0; 
                N_U1 =  N_CNT(D0/2,EG,kT,mu1-Uscf-deltaU/2)+N_CNT(D0/2,EG,kT,mu2-Uscf-deltaU/2)-N0; 
                CQ(2, kVg) = q*(N_U2-N_U1)/deltaU;
                %CQ(2, kVg)=q*(N_CNT(D0/2,EG,kT,mu1-Uscf+0.001*kT)-N_CNT(D0/2,EG,kT,mu1-Uscf-0.001*kT))/(0.002*kT);
                v_ave(kVg)=I(kV,kVg)/q/(dN+N0);
            end
        end
    end
end                     % Bias loop ends.

% Transconductance, gm and output conductance, gd and vinjare calculated at highest gate and drain biases
gm=(I(NV_Drain,NV_Gate)-I(NV_Drain,NV_Gate-1))/(V_Gate(NV_Gate)-V_Gate(NV_Gate-1));               % Transconductance gm = d(I)/d(Vg) with fixed Vd.
gd=(I(NV_Drain,NV_Gate)-I(NV_Drain-1,NV_Gate))/(V_Drain(NV_Drain)-V_Drain(NV_Drain-1));               % Output conductance gd = d(I)/d(Vd) with fixed Vg.
Av=gm/gd;
vinj=I(NV_Drain,NV_Gate)/(q*N(NV_Drain,NV_Gate));

% Calculate S and DIBL  %%%%%%%%%%%%%%%%%% added by Jing Wang 09/09/2003
% The subthreshold swing, S and the drain-induced barrier lowering, DIBL are calculated below. 
Ie1=log10(I(NV_Drain,:));                           % log10(I) for the highest drain Vd, used to evaluate S numerically.                            
vv1=interp1(Ie1,V_Gate,log10(I(NV_Drain,1)),'spline');   % vv1 is the corresponding Vg when I = I(Nv,1);
vv2=interp1(Ie1,V_Gate,log10(2*I(NV_Drain,1)),'spline'); % vv2 is the corresponding Vg when I = 2*I(Nv,1);
S=(vv2-vv1)/log10(2)*1000;                    % S is evaluated at low Vg and highest Vd, with the definition that S is the inverse slope of the
                                              % log10(I) vs. Vg curve, [mVolt/decade].
Ie2=log10(I(2,:));                            % log10(I) for the lowest drain bias Vd, used to evaluate DIBL numerically.
vv3=interp1(Ie2,V_Gate,log10(I(NV_Drain,1)),'spline');   % vv3 is the corresponding Vg in low drain bias when I = I(NV,1) at the highest drain bias.
DIBL=(vv3-vv1)/(V_Drain(NV_Drain)-V_Drain(2))*1000;             % DIBL is evaluated at the highest and lowest Vd at low gate bias Vg, [mVolt/Volt]
 
% Calculate channel conductance (cc) and transconductance (tc) as a
% function of Vgs %%%%%%%%%%%%%%%%%% added by Jing Wang 11/05/2003;updated
% by Yang Liu on 11/23/2003
cc=I(2,:)'./(V_Drain(2)-V_Drain(1));
tc=zeros(NV_Drain,1);
for i=2:NV_Gate
    tc(i)=(interp1(V_Gate,I(NV_Drain,:),V_Gate(i)+0.0005,'spline')-interp1(V_Gate,I(NV_Drain,:),V_Gate(i)-0.0005,'spline'))/0.001;
    %tc(i)=(exp(interp1(V,log(I(NV,:)),V(i)+0.0005))-exp(interp1(V,log(I(NV,:)),V(i)-0.0005)))/0.001;
end
tc(1)=(exp(interp1(V_Gate,log(I(NV_Drain,:)),V_Gate(1)+0.0005,'spline'))-exp(interp1(V_Gate,log(I(NV_Drain,:)),V_Gate(1)-0.0005,'spline')))/0.001;


% Series resistance Rs is treated below. Added by Yang Liu 11/23/2005

if Rs > 0
I=treatRs(V_Gate',V_Drain',I',Rs);
end


%%%%%%%%%%%%%%%  OUTPUT (Sayed Hasan: 5/25/2004)  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot_output;            % Plot results
save_charge;            % Save q*N
save_current;           % Save I
save_gm;                % Save gm values
save_uscf;              % Save Us
save_CQ;                % Save CQ
save_CQ_Cins;           % Save CQ Cins
save_v_ave;             % Save v_ave
write_results;          % Write results in results.m file
%save rawdata;           % save matlab variables
