function [I] = SiNWFETToy( t,d,epsr,mt,degen,T, NV_Gate,VI_Gate,VF_Gate,NV_Drain,VI_Drain,VF_Drain, Ef,alphag,alphad,Rs )

% function [I,V,Uscf,N] = SiNWFETToy( t,d,epsr,mt,T, VI,VF,NV, Ef,alphag,alphad )
%    Inputs:
%   ----------------
%       t - insulator thickness (m)
%       d - NT diameter (m)
%       epsr - insulator dielectric constant
%       mt - Transport Effective mass
%       T - Temp (K)
%
%       VI_Gate/VI_Drain - Initial Gate/Drain Voltage
%       VF_Gate/VF_Drain - Final Gate/Drain Voltage
%       NV_Gate/NV_Drain - # of Gate/Drain bias points
%
%       Ef - Fermi Level
%       alphag - gate control parameter
%       alphad - drain control parameter
%
%   Outputs:
%   ------------------
%       I = Current
%       V = Voltage
%       Uscf = Self Consistent Potential 
%       N = Free charge
%
% Based on FETToy Originally developed by : Anisur Rahman
%   Reference:
%   [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,
%   "Theory of Ballistic Nanotransistors", to appear in IEEE TED, 2003.
%
% Adapted for SiNWFETToy by : Jing Wang
% Latest Version Updated by : Sayed Hasan (05/24/2004)
%--------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Original Note by Anisur Rahman                                                  %% 
%%---------------------------------------------------------------------------------%% 
%% Name: FETToy                                                                    %% 
%% Written by Anisur Rahman, Purdue University, Dec 24, 2002                       %%
%% email: rahmana@purdue.edu                                                       %%
%% Routines used: input.m, plot_output.m, fermi.m, Uscf_zero.m, write_results.m    %%
%%                                                                                 %%
%% Reference:                                                                      %%
%% [1] A. Rahman, J. Guo, S. Datta, and M. Lundstrom,                              %%
%% "Theory of Ballistic Nanotransistors", to appear in IEEE TED, 2003.             %%
%%                                                                                 %%
%% Analytically calculates the ballistic I-V of a Double Gate ultra thin body      %%
%% MOSFET assuming that only the lowest unprimed subband is occupied (All          %%
%% constants are in MKS unit except energy, which is in eV)                        %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Physical constants
m0=0.91e-30;    hbar=1.05e-34;      q=1.6e-19;    eps0=8.85e-12;     kB=1.38e-23; 

m=mt*m0;                            % Transverse mass of electron
kT=kB*T/q;                          % Thermal voltage.
V_Gate=linspace(VI_Gate,VF_Gate,NV_Gate);               % Gate voltage steps.
V_Drain=linspace(VI_Drain,VF_Drain,NV_Drain);               % Gate voltage steps.
Cins=2*pi*epsr*eps0/log((t+d/2)/(d/2)); % Capacitance per unit length
CG=Cins;
C_SIG=CG/alphag;                    % C_SIG=sum of capacitors (see eq. (7b) in [1]).  
U0=q/C_SIG;                         % Charging energy (eq. (8b) in [1]).

% degeneracy is modified by Sayed Hasan (05/26/2004)
N1D= degen*m/hbar*(2*kT*q/pi/m)^0.5;         % Effective 1D density of states (include both +k and -k, valley degeneracy is degen)
% End degen. modification

N0 = N1D*fermi(Ef/kT,1,-1/2);         % Electron concentration at the top of the barrier in neutral device.
I0 = degen*(q*kB*T/pi/hbar);          % Valley degeneracy is given as input in "degen"

N=zeros(NV_Drain,NV_Gate);                     % Mobile charge density.
I=zeros(NV_Drain,NV_Gate);                     % Current.
Ef_mat=zeros(NV_Drain,NV_Gate);                % Source Fermi level.   
Esub_max=zeros(NV_Drain,NV_Gate);              % Energy at the top of the barrier.

for kVg=1:NV_Gate                                                        % Bias loop begins.
    for kV=1:NV_Drain
        Vg=V_Gate(kVg);  Vd=V_Drain(kV);
        mu1=Ef;   mu2=mu1-Vd;                                   % Source and drain fermi levels.
        UL=-(alphag*Vg)-(alphad*Vd);                            % Laplace potential.
        Uscf=fzero(@Uscf_zero,0,optimset('tolx',1e-12),N1D,mu1,mu2,kT,UL,U0,N0);
        dN=0.5*N1D*(fermi((mu1-Uscf)/kT,1,-1/2)+fermi((mu2-Uscf)/kT,1,-1/2))-N0;   % Mobile charge induced by gate and drain 
                                                                            
        N(kV,kVg)=dN;
        eta1=(mu1-Uscf)/kT; eta2=(mu2-Uscf)/kT;
        % Vd changes along fixed column and Vg changes along fixed row.
        I(kV,kVg)=I0*(fermi(eta1,1,0)-fermi(eta2,1,0)); 
        Esub_max(kV,kVg)=Uscf; Ef_mat(kV,kVg)=mu1; 
        
        % Added by Sayed Hasan 5/26/2004
        Us(kV,kVg) = Uscf;

        % Calculate Quantum Capacitance and average velocity %%%%%%%%%%%%%%%%%% added by Sayed Hasan 05/24/2004
        if ((kV==2)|(kV==NV_Drain)),
            deltaU = 0.001*kT;
            if (kV==2),
                N_U2 = 0.5*N1D*(fermi((mu1-Uscf+deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf+deltaU/2)/kT,1,-1/2))-N0;                
                N_U1 = 0.5*N1D*(fermi((mu1-Uscf-deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf-deltaU/2)/kT,1,-1/2))-N0;                
                CQ(1, kVg) = q*(N_U2-N_U1)/deltaU;
            elseif (kV==NV_Drain),
                N_U2 = 0.5*N1D*(fermi((mu1-Uscf+deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf+deltaU/2)/kT,1,-1/2))-N0;                
                N_U1 = 0.5*N1D*(fermi((mu1-Uscf-deltaU/2)/kT,1,-1/2)+fermi((mu2-Uscf-deltaU/2)/kT,1,-1/2))-N0;                
                CQ(2, kVg) = q*(N_U2-N_U1)/deltaU;
                v_ave(kVg)=I(kV,kVg)/q/(dN+N0);
            end
        end
    end
end                                                             % Bias loop ends.


% Transconductance, gm and output conductance, gd and vinjare calculated at highest gate and drain biases
gm=(I(NV_Drain,NV_Gate)-I(NV_Drain,NV_Gate-1))/(V_Gate(NV_Gate)-V_Gate(NV_Gate-1));               % Transconductance gm = d(I)/d(Vg) with fixed Vd.
gd=(I(NV_Drain,NV_Gate)-I(NV_Drain-1,NV_Gate))/(V_Drain(NV_Drain)-V_Drain(NV_Drain-1));               % Output conductance gd = d(I)/d(Vd) with fixed Vg.
Av=gm/gd;
vinj=I(NV_Drain,NV_Gate)/(q*N(NV_Drain,NV_Gate));

% Calculate S and DIBL  %%%%%%%%%%%%%%%%%% added by Jing Wang 09/09/2003
% The subthreshold swing, S and the drain-induced barrier lowering, DIBL are calculated below. 
Ie1=log10(I(NV_Drain,:));                           % log10(I) for the highest drain Vd, used to evaluate S numerically.                            
vv1=interp1(Ie1,V_Gate,log10(I(NV_Drain,1)),'spline');   % vv1 is the corresponding Vg when I = I(Nv,1);
vv2=interp1(Ie1,V_Gate,log10(2*I(NV_Drain,1)),'spline'); % vv2 is the corresponding Vg when I = 2*I(Nv,1);
S=(vv2-vv1)/log10(2)*1000;                    % S is evaluated at low Vg and highest Vd, with the definition that S is the inverse slope of the
                                              % log10(I) vs. Vg curve, [mVolt/decade].
Ie2=log10(I(2,:));                            % log10(I) for the lowest drain bias Vd, used to evaluate DIBL numerically.
vv3=interp1(Ie2,V_Gate,log10(I(NV_Drain,1)),'spline');   % vv3 is the corresponding Vg in low drain bias when I = I(NV,1) at the highest drain bias.
DIBL=(vv3-vv1)/(V_Drain(NV_Drain)-V_Drain(2))*1000;             % DIBL is evaluated at the highest and lowest Vd at low gate bias Vg, [mVolt/Volt]

% Series resistance Rs is treated below. Added by Yang Liu 11/23/2005
if Rs > 0
I=treatRs(V_Gate',V_Drain',I',Rs);
end

%%%%%%%%%%%%%%%  OUTPUT (Sayed Hasan: 5/25/2004)  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot_output;            % Plot results
save_charge;            % Save q*N
save_current;           % Save I
save_gm;                % Save gm values
save_uscf;              % Save Us
save_CQ;                % Save CQ
save_CQ_Cins;           % Save CQ Cins
save_v_ave;             % Save v_ave
write_results;          % Write results in results.m file
%save rawdata;           % save matlab variables
